@@ -7,12 +7,14 @@
 
 /**
  * @packageDocumentation
- * @module aws-iot-device-sdk
+ * @module jobs
  */
 
 import * as model from "./model";
-import { mqtt } from "aws-crt";
+import { mqtt, mqtt5 } from "aws-crt";
 import { TextDecoder } from "util";
+import * as service_client_mqtt_adapter from "../service_client_mqtt_adapter";
+
 export { model };
 
 /**
@@ -44,6 +46,9 @@ export class IotJobsError extends Error {
  */
 export class IotJobsClient {
 
+    // @ts-ignore
+    private mqttAdapter: service_client_mqtt_adapter.IServiceClientMqttAdapter;
+
     private decoder = new TextDecoder('utf-8');
 
     private static INVALID_PAYLOAD_PARSING_ERROR = "Invalid/unknown error parsing payload into response";
@@ -56,7 +61,33 @@ export class IotJobsClient {
         }
     }
 
-    constructor(private connection: mqtt.MqttClientConnection) {
+    constructor(connection?: mqtt.MqttClientConnection) {
+        if (connection !== undefined) {
+           this.mqttAdapter = new service_client_mqtt_adapter.ServiceClientMqtt311Adapter(connection);
+        }
+    }
+
+    /**
+     * Creates a new IotJobsClient that uses the SDK Mqtt5 client internally.
+     *
+     * The pre-existing constructor that is bound to the MQTT311 client makes this awkward since we
+     * must support
+     *
+     * ```
+     * new IotJobsClient(mqtt311connection);
+     * ```
+     *
+     * for backwards compatibility, but still want to be able to inject an MQTT5 client as well.
+     *
+     * @param client the MQTT5 client to use with this service client
+     *
+     * @returns a new IotJobsClient instance
+     */
+    static newFromMqtt5Client(client: mqtt5.Mqtt5Client) : IotJobsClient {
+        let serviceClient: IotJobsClient = new IotJobsClient();
+        serviceClient.mqttAdapter = new service_client_mqtt_adapter.ServiceClientMqtt5Adapter(client);
+
+        return serviceClient;
     }
 
     /**
@@ -104,7 +135,7 @@ export class IotJobsClient {
             }
         }
 
-        return this.connection.subscribe(topic, qos, on_message);
+        return this.mqttAdapter.subscribe(topic, qos, on_message);
     }
 
     /**
@@ -152,7 +183,7 @@ export class IotJobsClient {
             }
         }
 
-        return this.connection.subscribe(topic, qos, on_message);
+        return this.mqttAdapter.subscribe(topic, qos, on_message);
     }
 
     /**
@@ -201,7 +232,7 @@ export class IotJobsClient {
             }
         }
 
-        return this.connection.subscribe(topic, qos, on_message);
+        return this.mqttAdapter.subscribe(topic, qos, on_message);
     }
 
     /**
@@ -249,7 +280,7 @@ export class IotJobsClient {
             }
         }
 
-        return this.connection.subscribe(topic, qos, on_message);
+        return this.mqttAdapter.subscribe(topic, qos, on_message);
     }
 
     /**
@@ -298,7 +329,7 @@ export class IotJobsClient {
             }
         }
 
-        return this.connection.subscribe(topic, qos, on_message);
+        return this.mqttAdapter.subscribe(topic, qos, on_message);
     }
 
     /**
@@ -347,7 +378,7 @@ export class IotJobsClient {
             }
         }
 
-        return this.connection.subscribe(topic, qos, on_message);
+        return this.mqttAdapter.subscribe(topic, qos, on_message);
     }
 
     /**
@@ -376,7 +407,7 @@ export class IotJobsClient {
         let topic: string = "$aws/things/{thingName}/jobs/{jobId}/update";
         topic = topic.replace("{thingName}", request.thingName);
         topic = topic.replace("{jobId}", request.jobId);
-        return this.connection.publish(topic, JSON.stringify(request), qos);
+        return this.mqttAdapter.publish(topic, JSON.stringify(request), qos);
     }
 
     /**
@@ -425,7 +456,7 @@ export class IotJobsClient {
             }
         }
 
-        return this.connection.subscribe(topic, qos, on_message);
+        return this.mqttAdapter.subscribe(topic, qos, on_message);
     }
 
     /**
@@ -453,7 +484,7 @@ export class IotJobsClient {
 
         let topic: string = "$aws/things/{thingName}/jobs/get";
         topic = topic.replace("{thingName}", request.thingName);
-        return this.connection.publish(topic, JSON.stringify(request), qos);
+        return this.mqttAdapter.publish(topic, JSON.stringify(request), qos);
     }
 
     /**
@@ -501,7 +532,7 @@ export class IotJobsClient {
             }
         }
 
-        return this.connection.subscribe(topic, qos, on_message);
+        return this.mqttAdapter.subscribe(topic, qos, on_message);
     }
 
     /**
@@ -549,7 +580,7 @@ export class IotJobsClient {
             }
         }
 
-        return this.connection.subscribe(topic, qos, on_message);
+        return this.mqttAdapter.subscribe(topic, qos, on_message);
     }
 
     /**
@@ -597,7 +628,7 @@ export class IotJobsClient {
             }
         }
 
-        return this.connection.subscribe(topic, qos, on_message);
+        return this.mqttAdapter.subscribe(topic, qos, on_message);
     }
 
     /**
@@ -625,7 +656,7 @@ export class IotJobsClient {
 
         let topic: string = "$aws/things/{thingName}/jobs/start-next";
         topic = topic.replace("{thingName}", request.thingName);
-        return this.connection.publish(topic, JSON.stringify(request), qos);
+        return this.mqttAdapter.publish(topic, JSON.stringify(request), qos);
     }
 
     /**
@@ -654,7 +685,7 @@ export class IotJobsClient {
         let topic: string = "$aws/things/{thingName}/jobs/{jobId}/get";
         topic = topic.replace("{thingName}", request.thingName);
         topic = topic.replace("{jobId}", request.jobId);
-        return this.connection.publish(topic, JSON.stringify(request), qos);
+        return this.mqttAdapter.publish(topic, JSON.stringify(request), qos);
     }
 
 }
