@@ -7,12 +7,14 @@
 
 /**
  * @packageDocumentation
- * @module aws-iot-device-sdk
+ * @module shadow
  */
 
 import * as model from "./model";
-import { mqtt } from "aws-crt";
+import { mqtt, mqtt5 } from "aws-crt";
 import { TextDecoder } from "util";
+import * as service_client_mqtt_adapter from "../service_client_mqtt_adapter";
+
 export { model };
 
 /**
@@ -44,6 +46,9 @@ export class IotShadowError extends Error {
  */
 export class IotShadowClient {
 
+    // @ts-ignore
+    private mqttAdapter: service_client_mqtt_adapter.IServiceClientMqttAdapter;
+
     private decoder = new TextDecoder('utf-8');
 
     private static INVALID_PAYLOAD_PARSING_ERROR = "Invalid/unknown error parsing payload into response";
@@ -56,7 +61,33 @@ export class IotShadowClient {
         }
     }
 
-    constructor(private connection: mqtt.MqttClientConnection) {
+    constructor(connection?: mqtt.MqttClientConnection) {
+        if (connection !== undefined) {
+           this.mqttAdapter = new service_client_mqtt_adapter.ServiceClientMqtt311Adapter(connection);
+        }
+    }
+
+    /**
+     * Creates a new IotShadowClient that uses the SDK Mqtt5 client internally.
+     *
+     * The pre-existing constructor that is bound to the MQTT311 client makes this awkward since we
+     * must support
+     *
+     * ```
+     * new IotShadowClient(mqtt311connection);
+     * ```
+     *
+     * for backwards compatibility, but still want to be able to inject an MQTT5 client as well.
+     *
+     * @param client the MQTT5 client to use with this service client
+     *
+     * @returns a new IotShadowClient instance
+     */
+    static newFromMqtt5Client(client: mqtt5.Mqtt5Client) : IotShadowClient {
+        let serviceClient: IotShadowClient = new IotShadowClient();
+        serviceClient.mqttAdapter = new service_client_mqtt_adapter.ServiceClientMqtt5Adapter(client);
+
+        return serviceClient;
     }
 
     /**
@@ -104,7 +135,7 @@ export class IotShadowClient {
             }
         }
 
-        return this.connection.subscribe(topic, qos, on_message);
+        return this.mqttAdapter.subscribe(topic, qos, on_message);
     }
 
     /**
@@ -152,7 +183,7 @@ export class IotShadowClient {
             }
         }
 
-        return this.connection.subscribe(topic, qos, on_message);
+        return this.mqttAdapter.subscribe(topic, qos, on_message);
     }
 
     /**
@@ -201,7 +232,7 @@ export class IotShadowClient {
             }
         }
 
-        return this.connection.subscribe(topic, qos, on_message);
+        return this.mqttAdapter.subscribe(topic, qos, on_message);
     }
 
     /**
@@ -250,7 +281,7 @@ export class IotShadowClient {
             }
         }
 
-        return this.connection.subscribe(topic, qos, on_message);
+        return this.mqttAdapter.subscribe(topic, qos, on_message);
     }
 
     /**
@@ -278,7 +309,7 @@ export class IotShadowClient {
 
         let topic: string = "$aws/things/{thingName}/shadow/delete";
         topic = topic.replace("{thingName}", request.thingName);
-        return this.connection.publish(topic, JSON.stringify(request), qos);
+        return this.mqttAdapter.publish(topic, JSON.stringify(request), qos);
     }
 
     /**
@@ -307,7 +338,7 @@ export class IotShadowClient {
         let topic: string = "$aws/things/{thingName}/shadow/name/{shadowName}/get";
         topic = topic.replace("{shadowName}", request.shadowName);
         topic = topic.replace("{thingName}", request.thingName);
-        return this.connection.publish(topic, JSON.stringify(request), qos);
+        return this.mqttAdapter.publish(topic, JSON.stringify(request), qos);
     }
 
     /**
@@ -355,7 +386,7 @@ export class IotShadowClient {
             }
         }
 
-        return this.connection.subscribe(topic, qos, on_message);
+        return this.mqttAdapter.subscribe(topic, qos, on_message);
     }
 
     /**
@@ -403,7 +434,7 @@ export class IotShadowClient {
             }
         }
 
-        return this.connection.subscribe(topic, qos, on_message);
+        return this.mqttAdapter.subscribe(topic, qos, on_message);
     }
 
     /**
@@ -452,7 +483,7 @@ export class IotShadowClient {
             }
         }
 
-        return this.connection.subscribe(topic, qos, on_message);
+        return this.mqttAdapter.subscribe(topic, qos, on_message);
     }
 
     /**
@@ -501,7 +532,7 @@ export class IotShadowClient {
             }
         }
 
-        return this.connection.subscribe(topic, qos, on_message);
+        return this.mqttAdapter.subscribe(topic, qos, on_message);
     }
 
     /**
@@ -549,7 +580,7 @@ export class IotShadowClient {
             }
         }
 
-        return this.connection.subscribe(topic, qos, on_message);
+        return this.mqttAdapter.subscribe(topic, qos, on_message);
     }
 
     /**
@@ -578,7 +609,7 @@ export class IotShadowClient {
         let topic: string = "$aws/things/{thingName}/shadow/name/{shadowName}/delete";
         topic = topic.replace("{shadowName}", request.shadowName);
         topic = topic.replace("{thingName}", request.thingName);
-        return this.connection.publish(topic, JSON.stringify(request), qos);
+        return this.mqttAdapter.publish(topic, JSON.stringify(request), qos);
     }
 
     /**
@@ -627,7 +658,7 @@ export class IotShadowClient {
             }
         }
 
-        return this.connection.subscribe(topic, qos, on_message);
+        return this.mqttAdapter.subscribe(topic, qos, on_message);
     }
 
     /**
@@ -675,7 +706,7 @@ export class IotShadowClient {
             }
         }
 
-        return this.connection.subscribe(topic, qos, on_message);
+        return this.mqttAdapter.subscribe(topic, qos, on_message);
     }
 
     /**
@@ -723,7 +754,7 @@ export class IotShadowClient {
             }
         }
 
-        return this.connection.subscribe(topic, qos, on_message);
+        return this.mqttAdapter.subscribe(topic, qos, on_message);
     }
 
     /**
@@ -751,7 +782,7 @@ export class IotShadowClient {
 
         let topic: string = "$aws/things/{thingName}/shadow/update";
         topic = topic.replace("{thingName}", request.thingName);
-        return this.connection.publish(topic, JSON.stringify(request), qos);
+        return this.mqttAdapter.publish(topic, JSON.stringify(request), qos);
     }
 
     /**
@@ -779,7 +810,7 @@ export class IotShadowClient {
 
         let topic: string = "$aws/things/{thingName}/shadow/get";
         topic = topic.replace("{thingName}", request.thingName);
-        return this.connection.publish(topic, JSON.stringify(request), qos);
+        return this.mqttAdapter.publish(topic, JSON.stringify(request), qos);
     }
 
     /**
@@ -827,7 +858,7 @@ export class IotShadowClient {
             }
         }
 
-        return this.connection.subscribe(topic, qos, on_message);
+        return this.mqttAdapter.subscribe(topic, qos, on_message);
     }
 
     /**
@@ -876,7 +907,7 @@ export class IotShadowClient {
             }
         }
 
-        return this.connection.subscribe(topic, qos, on_message);
+        return this.mqttAdapter.subscribe(topic, qos, on_message);
     }
 
     /**
@@ -905,7 +936,7 @@ export class IotShadowClient {
         let topic: string = "$aws/things/{thingName}/shadow/name/{shadowName}/update";
         topic = topic.replace("{shadowName}", request.shadowName);
         topic = topic.replace("{thingName}", request.thingName);
-        return this.connection.publish(topic, JSON.stringify(request), qos);
+        return this.mqttAdapter.publish(topic, JSON.stringify(request), qos);
     }
 
     /**
@@ -954,7 +985,7 @@ export class IotShadowClient {
             }
         }
 
-        return this.connection.subscribe(topic, qos, on_message);
+        return this.mqttAdapter.subscribe(topic, qos, on_message);
     }
 
     /**
@@ -1003,7 +1034,7 @@ export class IotShadowClient {
             }
         }
 
-        return this.connection.subscribe(topic, qos, on_message);
+        return this.mqttAdapter.subscribe(topic, qos, on_message);
     }
 
 }
